﻿using C1.Win.C1FlexGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CrossHair_C1FlexGrid
{
    public partial class Form1 : Form
    {
        DataTable dt;
        bool rowEntered;
        bool colEntered;
        int row;
        int col;
        int prevRow;
        int prevCol;
         SolidBrush _bdrBrush;
        public Form1()
        {
            InitializeComponent();

            #region "Creating data for C1FlexGrid"
            dt = new DataTable();
            for (int c = 0; c <= 100; c++)
            {
                dt.Columns.Add("Column" + c.ToString());
            }
            for(int r=1;r<10000;r++)
            {
                var row = dt.Rows.Add();
                    
                for (int c = 0; c <= 100; c++)
                {
                    row[c] = "Value[" + r.ToString() + "," + c.ToString() + "]";
                }
            }
#endregion

            rowEntered = false;
            colEntered = false;
            prevRow= -1;
            prevCol= -1;

            _bdrBrush = new SolidBrush(Color.Red);

            c1FlexGrid1.CellChanged += C1FlexGrid1_CellChanged;
            c1FlexGrid1.DrawMode = C1.Win.C1FlexGrid.DrawModeEnum.OwnerDraw;
            c1FlexGrid1.OwnerDrawCell += C1FlexGrid1_OwnerDrawCell;
        }

        private void C1FlexGrid1_OwnerDrawCell(object sender, C1.Win.C1FlexGrid.OwnerDrawCellEventArgs e)
        {
            if(!e.Measuring)
            {
                // we only want cells with our style  
                CellStyle s = c1FlexGrid1.GetCellStyle(e.Row, e.Col);
                if (s == null || (s.Name != "HorizontalBorder" && s.Name != "VerticalBorder" && s.Name!= "MergedBorder"))
                {
                    return;
                }

                // draw cell content as usual
                e.DrawCell();

                //if cell resides in row whose cell changed
                if (s.Name == "HorizontalBorder")
                {
                    Margins marginHz = new Margins(0,0,3,0);
                    Rectangle rec;
                    Graphics gr = e.Graphics;
                    rec = e.Bounds;
                    rec.Height = marginHz.Top;
                    gr.FillRectangle(_bdrBrush, rec);
                    prevRow = e.Row;

                }
                //if cell resides in column whose cell changed
                else if (s.Name == "VerticalBorder")
                {
                    Margins marginVr = new Margins(3, 0, 0,0);
                    Rectangle rect;
                    Graphics grs = e.Graphics;
                    rect = e.Bounds;
                    rect.Width = marginVr.Left;
                    grs.FillRectangle(_bdrBrush, rect);
                    prevCol = e.Col;
                }
                //if cell resides in row and column (both) whose cell changed
                else if (s.Name== "MergedBorder")
                {
                    Margins marginMrg = new Margins(3, 0, 3,0);
                    Rectangle rctTop;
                    Graphics grs = e.Graphics;
                    rctTop = e.Bounds;
                    rctTop.Height = marginMrg.Top;
                    grs.FillRectangle(_bdrBrush, rctTop);

                    Rectangle rctLeft;
                    rctLeft = e.Bounds;
                    rctLeft.Width = marginMrg.Left;
                    grs.FillRectangle(_bdrBrush, rctLeft);
                }
               
            }
        }

        private void C1FlexGrid1_CellChanged(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            if (prevRow > -1)
            {
                CellRange prevRowRange = c1FlexGrid1.GetCellRange(prevRow, 1, prevRow, c1FlexGrid1.Cols.Count - 1);
                prevRowRange.Style = null;
                prevRow = -1;
            }
            if(prevCol>-1)
            {
                CellRange prevColRange = c1FlexGrid1.GetCellRange(1, prevCol, c1FlexGrid1.Rows.Count - 1, prevCol);
                prevColRange.Style = null;
                prevCol = -1;

            }
            CellRange colRange = c1FlexGrid1.GetCellRange(1,col,c1FlexGrid1.Rows.Count-1,col);  //getting all cells in modified column
            colRange.Style = c1FlexGrid1.Styles["VerticalBorder"];
            CellRange rowRange= c1FlexGrid1.GetCellRange(row,  1, row, c1FlexGrid1.Cols.Count - 1);  //getting all cells in modified row
            rowRange.Style = c1FlexGrid1.Styles["HorizontalBorder"];
            CellStyle mergedStyle = c1FlexGrid1.Styles["MergedBorder"];
            c1FlexGrid1.SetCellStyle(e.Row, e.Col, mergedStyle);
            c1FlexGrid1.Select(e.Row, e.Col);

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            c1FlexGrid1.DataSource = dt;
            row = 0;
            col = 0;

            CellStyle vStyle = c1FlexGrid1.Styles.Add("VerticalBorder"); //adding Vertical style to FlexGrid
            CellStyle hStyle = c1FlexGrid1.Styles.Add("HorizontalBorder");  //adding Horizontal style to FlexGrid
            CellStyle mergedStyle = c1FlexGrid1.Styles.Add("MergedBorder"); // adding style where Vertical and Horizontal style cross
        }

        #region "Code for properly taking row, column index and enabling/disabling control taking new value"
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            if(textBox1.Text.Length>0)
            {
                if(!rowEntered)
                {
                    rowEntered = true;
                }
            }
            else
            {
                rowEntered = false;
            }
            EnableNewValueControls();
            if(textBox1.Text!="")
            {
                row = Convert.ToInt32(textBox1.Text);
                if (Convert.ToInt32(textBox1.Text) > 10000)
                {
                    MessageBox.Show("Row index are from 1-10,000");
                }
            }
        }

        private void textBox2_TextChanged(object sender, EventArgs e)
        {
            if (textBox2.Text.Length > 0)
            {
                colEntered = true;
            }
            else
            {
                colEntered = false;
            }
            if (textBox2.Text != "")
            {
                col = Convert.ToInt32(textBox2.Text);

                if (Convert.ToInt32(textBox2.Text) > 100)
                {
                    MessageBox.Show("Column index are from 1-100");
                }
            }

            EnableNewValueControls();
        }

        //allow entering new value only after row and col indexes have been entered
        private void EnableNewValueControls()
        {
            if (rowEntered && colEntered)
            {
                label3.Enabled = true;
                textBox3.Enabled = true;
            }
            else
            {
                label3.Enabled = false;
                textBox3.Enabled = false;
            }
        }

        // allowing only integer values, since row index is of type Integer
        private void textBox1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsControl(e.KeyChar) && !char.IsDigit(e.KeyChar))
            {
                e.Handled = true;
            }
        }

        // allowing only integer values, since col index is of type Integer
        private void textBox2_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsControl(e.KeyChar) && !char.IsDigit(e.KeyChar))
            {
                e.Handled = true;
            }
        }

        //insert new value in Flexgrid
        private void textBox3_TextChanged(object sender, EventArgs e)
        {
            c1FlexGrid1[row, col] = textBox3.Text;
        }
#endregion
    }
}
